package net.knarcraft.armoredelytra.listener;

import net.knarcraft.armoredelytra.metadata.DurabilityManager;
import net.knarcraft.armoredelytra.metadata.NBTEditor;
import net.knarcraft.armoredelytra.property.ArmorTier;
import net.knarcraft.armoredelytra.util.GenericUtil;
import org.bukkit.Material;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerItemDamageEvent;

public class FlyDurabilityListener implements Listener {
    private final boolean disableDurability;
    private final NBTEditor nbtEditor;
    private final DurabilityManager durabilityManager;

    public FlyDurabilityListener(boolean disableDurability, NBTEditor nbtEditor, DurabilityManager durabilityManager) {
        this.disableDurability = disableDurability;
        this.nbtEditor = nbtEditor;
        this.durabilityManager = durabilityManager;
    }

    // Do not decrease elytra durability while flying.
    @EventHandler(priority = EventPriority.LOWEST)
    public void onItemDamage(PlayerItemDamageEvent e) {
        if (e.getItem().getType() != Material.ELYTRA) {
            return;
        }

        if (!e.getPlayer().isGliding()) {
            return;
        }

        final ArmorTier armorTier = nbtEditor.getArmorTier(e.getItem());
        if (armorTier == ArmorTier.NONE) {
            return;
        }

        // This also cancels durability decrease when it should (i.e. getting hit) while flying,
        // but that is likely to be rare enough for it to not matter.
        e.setCancelled(true);
        if (disableDurability) {
            return;
        }

        final int newDurability = durabilityManager.removeDurability(e.getItem(), e.getDamage(), armorTier);
        if (durabilityManager.isBroken(newDurability, armorTier)) {
            GenericUtil.moveChestplateToInventory(e.getPlayer());
        }
    }
}
