package net.knarcraft.armoredelytra.update;

/**
 * A constant reason for the result of {@link UpdateResult}.
 */
public enum UpdateReason {

    /**
     * A new update is available for download.
     * <p>
     * This is the only reason that requires an update.
     */
    NEW_UPDATE,

    /**
     * A successful connection to the SpiGet API could not be established.
     */
    COULD_NOT_CONNECT,

    /**
     * The JSON retrieved from SpiGet was invalid or malformed.
     */
    INVALID_JSON,

    /**
     * A 401 error was returned by the SpiGet API.
     */
    UNAUTHORIZED_QUERY,

    /**
     * The version of the plugin installed on the server is greater than the one uploaded to SpigotMC's resources
     * section.
     */
    UNRELEASED_VERSION,

    /**
     * An unknown error occurred.
     */
    UNKNOWN_ERROR,

    /**
     * The plugin uses an unsupported version scheme, therefore a proper comparison between versions could not be
     * made.
     */
    UNSUPPORTED_VERSION_SCHEME,

    /**
     * The plugin is up-to-date with the version released on SpigotMC's resources section.
     */
    UP_TO_DATE

}
