package net.knarcraft.armoredelytra.update;

import org.apache.commons.lang.math.NumberUtils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * A functional interface to compare two version Strings with similar version schemes.
 */
public class VersionScheme {

    private static final Pattern DECIMAL_SCHEME_PATTERN = Pattern.compile("\\d+(?:\\.\\d+)*");

    /**
     * Compare two versions and return the higher of the two. If null is returned, it is assumed that at least one
     * of the two versions are unsupported by this version scheme parser.
     *
     * @param first  the first version to check
     * @param second the second version to check
     * @return the greater of the two versions. null if unsupported version schemes
     */
    String compareVersions(String first, String second) {
        String[] firstSplit = splitVersionInfo(first), secondSplit = splitVersionInfo(second);
        if (firstSplit == null || secondSplit == null) {
            return null;
        }

        for (int i = 0; i < Math.min(firstSplit.length, secondSplit.length); i++) {
            int currentValue = NumberUtils.toInt(firstSplit[i]), newestValue = NumberUtils.toInt(secondSplit[i]);

            if (newestValue > currentValue) {
                return second;
            } else if (newestValue < currentValue) {
                return first;
            }
        }

        return (secondSplit.length > firstSplit.length) ? second : first;
    }

    private static String[] splitVersionInfo(String version) {
        Matcher matcher = DECIMAL_SCHEME_PATTERN.matcher(version);
        if (!matcher.find()) {
            return null;
        }

        return matcher.group().split("\\.");
    }

}
